local frame = CreateFrame("Frame"); -- Need a frame to respond to events
EVENTS_TO_REGISTER = {"ADDON_LOADED", "COMBAT_LOG_EVENT_UNFILTERED", "PLAYER_ENTERING_WORLD", "PLAYER_LOGOUT", "CHAT_MSG_ADDON", "ENCOUNTER_START", "ENCOUNTER_END", "PLAYER_REGEN_DISABLED", "PLAYER_REGEN_ENABLED"}
for _, v in pairs(EVENTS_TO_REGISTER) do
	frame:RegisterEvent(v)
end

-- Addon
local hf = CreateFrame("Frame",modName,UIParent);
loaded = false

_G.HardFinger = {}
local HardFinger = _G.HardFinger
local playerName = UnitName("player")
HardFinger.HardFinger_Config = HardFinger.HardFinger_Config or {}
local P = HardFinger.HardFinger_Config

COMBATLOG_OBJECT_TYPE_PET		= COMBATLOG_OBJECT_TYPE_PET or 0x00001000
COMBATLOG_OBJECT_TYPE_GUARDIAN		= COMBATLOG_OBJECT_TYPE_GUARDIAN or 0x00002000
COMBATLOG_OBJECT_TYPE_NPC 		= COMBATLOG_OBJECT_TYPE_NPC or 0x0000800
PET_FILTER_BITMASK			= COMBATLOG_OBJECT_TYPE_PET + COMBATLOG_OBJECT_TYPE_GUARDIAN + COMBATLOG_OBJECT_TYPE_NPC
COMBATLOG_OBJECT_REACTION_FRIENDLY	= COMBATLOG_OBJECT_REACTION_FRIENDLY or 0x00000010


function HardFinger.UpdateGUIVariables()
	HardFinger.DPrint("HardFinger - GUI Options Updated")

	HardFinger.ListOfChannels = {}
	HardFinger.ListOfChannels["raid/party/instance"] = 1
	if IsInGuild() then
			HardFinger.ListOfChannels["/g GuildChat"] = "GUILD_CHANNEL"
	end
	for i=5,15,1 do
		a,b = GetChannelName(i)
		if a ~= 0 and b ~= 0 then
			HardFinger.ListOfChannels[a .. ". " .. b] = a
		end
	end
end

function HardFinger.ForceLoad(updateOnly)
	if updateOnly ~= true then
		HFDB = HFDB or { }

		HardFinger.HardFinger_Config.Profiles = HFDB

		HFDB.Default = { }
		HFDB.Default.Name = "Default"
		HardFinger.DefaultVariables = {"Status", "WhoPulled", "WhoPulledChannel", "Taunt", "BloodLust", "BattleRez", "TimeToDie", "ReportChannel", "Interrupt"}
		for _, v in pairs(HardFinger.DefaultVariables) do
			HFDB.Default[v] = false
		end

		HFDB.Default["EncountersOptions"] = {}
		for bossName, bossTable in pairs(Encounters_Options) do
			HFDB.Default["EncountersOptions"][bossName] = {}
			for id, bossAbilitiesTable in pairs(bossTable) do
				varName = bossAbilitiesTable.var
				HFDB.Default["EncountersOptions"][bossName][varName] = false
			end
		end
	
		-- Does the player has a valid active Profile selected?
		if HFDB[playerName] == nil or HFDB[playerName]["EncountersOptions"] == nil then
			HFDB[playerName] = HFDB.Default
		end
		
		-- Does the player has a valid active encounter options table?
		HFDB[playerName]["EncountersOptions"] = HFDB[playerName]["EncountersOptions"] or {}
		for bossName, bossTable in pairs(Encounters_Options) do
			HFDB[playerName]["EncountersOptions"][bossName] = HFDB[playerName]["EncountersOptions"][bossName] or {}
			for id, bossAbilitiesTable in pairs(bossTable) do
				varName = bossAbilitiesTable.var
				HFDB[playerName]["EncountersOptions"][bossName][varName] = HFDB[playerName]["EncountersOptions"][bossName][varName] or false
			end
		end
		
		loaded = true	
	end


	for _, v in pairs(HardFinger.DefaultVariables) do
		HardFinger.HardFinger_Config[v] = HFDB[playerName][v]
	end
	
	HardFinger.HardFinger_Config.EncountersOptions = HardFinger.TableHardCopy(HFDB[playerName].EncountersOptions)

	if (HardFingerOptions:IsShown()) then
		HardFingerOptions:Hide();
		HardFingerOptions:Show();
	end
end


function HardFinger.SaveVariables()
	for _, v in pairs(HardFinger.DefaultVariables) do
		HFDB[playerName][v] = HardFinger.HardFinger_Config[v]
	end
	HFDB[playerName].EncountersOptions = HardFinger.TableHardCopy(HardFinger.HardFinger_Config.EncountersOptions)
	HardFinger.ForceLoad(true)
end

function HardFinger.ResetProfile()
	HardFinger.DPrint("HardFinger - Profile: "..playerName.." reseted.")
	for _, v in pairs(HardFinger.DefaultVariables) do
		HFDB[playerName][v] = false
	end
	HFDB[playerName]["EncountersOptions"] = {}
	for bossName, bossTable in pairs(Encounters_Options) do
		HFDB[playerName]["EncountersOptions"][bossName] = {}
		for id, bossAbilitiesTable in pairs(bossTable) do
			
			varName = bossAbilitiesTable.var
			HardFinger.DPrint(bossName, varName)
			HFDB[playerName]["EncountersOptions"][bossName][varName] = false
		end
	end
	HardFinger.ForceLoad(true)
end


--
local function OnEvent(self, event, ...)
	if event == "COMBAT_LOG_EVENT_UNFILTERED" and P.Status == true then
		_, message, _, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags, spellID, spellName, param12, param13, param14, param15, _, _, param18 = ...
		-- Always running Display failTable time
		if HardFinger.FailTableDelayTimer ~= nil then
			HardFinger.DisplayFailTable()
		end


		if message == "SPELL_SUMMON" then
			if bit.band(destFlags, PET_FILTER_BITMASK) ~= 0 then
				HardFinger.PetOwnersTable[destGUID] = {sourceName, sourceGUID}
			end
		end

		if P.WhoPulled == true then
			HardFinger.WhoPulled("ALL", false, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
		end
		
		-- Functions WhoTaunted, WhoREzzed and WhoLusted
		HardFinger.TauntRezLust(message, sourceName, destName, destRaidFlags, spellID)

		----------------------------------------------------------------------------------------------------------
		--						NIGHTHOLD BOSSES					--
		----------------------------------------------------------------------------------------------------------
			-- Skorpyron
		if HardFinger.ActiveEncounter == 1849 then
			if P["EncountersOptions"]["Skorpyron"]["trackWhoPulled"] == true then
				HardFinger.WhoPulled("Skorpyron", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
			end
			if P["EncountersOptions"]["Skorpyron"]["trackShockwave"] == true then
				HardFinger.BadSpellHitID(210074, 600000, "ALL", P["EncountersOptions"]["Skorpyron"]["trackShockwaveChat"], true, message, destName, destFlags, spellID, param13, param14, param18)-- Shockwave 210074
			end
			if P["EncountersOptions"]["Skorpyron"]["trackArcanoslashDamage"] == true then
				HardFinger.BadSpellDamageID(204275, 0, "Healer|Damager", false, true, message, destName, spellID, param13, param14, param18)-- Arcanoslash
			end
			if P["EncountersOptions"]["Skorpyron"]["trackArcanoslashDeaths"] == true then
				HardFinger.BadSpellHitIDDeath(204275, 0, "Healer|Damager", false, true, message, destName, spellID, param13, param14, param18)-- Arcanoslash
			end
			if P["EncountersOptions"]["Skorpyron"]["trackBlast"] == true then
				HardFinger.BadSpellHitID(204483, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param18)-- Focused Blast 204483
				HardFinger.BadSpellHitIDDeath(204483, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param18)-- Focused Blast 204483
			end
			if P["EncountersOptions"]["Skorpyron"]["trackChitin"] == true then
				HardFinger.BadSpellDamageID(204744, 0, "ALL", false, true, message, destName, spellID, param13, param14, param18)-- Green Toxic Pool
				HardFinger.BadSpellDamageID(204744, 0, "ALL", false, true, message, destName, spellID, param13, param14, param18)-- Green Toxic Pool
			end
			if P["EncountersOptions"]["Skorpyron"]["trackRedCrystalPulseDamage"] == true then
				HardFinger.BadSpellDamageID(214662, 0, "ALL", false, true, sourceName, destName, spellID, param13, param14, param18)-- Red pulsing
				HardFinger.BadSpellDamageID(211801, 0, "ALL", false, true, sourceName, destName, spellID, param13, param14, param18)-- Red pulsing
			end
			if P["EncountersOptions"]["Skorpyron"]["trackRedCrystalPulseDeaths"] == true then
				HardFinger.BadSpellHitIDDeath(214662, 0, "ALL", false, true, sourceName, destName, spellID, param13, param14, param18)-- Red pulsing
				HardFinger.BadSpellHitIDDeath(211801, 0, "ALL", false, true, sourceName, destName, spellID, param13, param14, param18)-- Red pulsing
			end
			if P["EncountersOptions"]["Skorpyron"]["trackRedCrystalExplosionDamage"] == true then
				HardFinger.BadSpellDamageID(226229, 0, "ALL", false, true, sourceName, destName, spellID, param13, param14, param18)-- Red crystal explosion // Volatile Resonance
			end
			if P["EncountersOptions"]["Skorpyron"]["trackRedCrystalExplosionDeaths"] == true then
				HardFinger.BadSpellHitIDDeath(226229, 0, "ALL", false, true, sourceName, destName, spellID, param13, param14, param18)-- Red crystal explosion // Volatile Resonance
			end
			
			-- Chronomatic Anomaly / ID: 
		elseif HardFinger.ActiveEncounter == 1865 then
			if P["EncountersOptions"]["Chronomatic Anomaly"]["trackWhoPulled"] == true then
				HardFinger.WhoPulled("Chronomatic Anomaly", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
			end
			if P["EncountersOptions"]["Chronomatic Anomaly"]["trackTimeReleaseHealing"] == true then
				HardFinger.HealAbsorbed(message, "Time Release", sourceName, destName, param15)
			end
			if HardFinger.Hpp("boss1") < 75 then
				HardFinger.BadSpreadDamageID(206615, "a", 0, "Damager|Healer", false, true, sourceName, destName, spellID, param13, param14, param18)
				HardFinger.BadSpellAuraID(206617, -1, true, false, true, message, spellID, destName, param14)-- Time Bomb Applications
			end
			if P["EncountersOptions"]["Chronomatic Anomaly"]["trackDamageDoneAdd"] == true and HardFinger.Hpp("boss1") < 30 then
				HardFinger.DamageToTarget("Waning Time Particle", "Last Set of Adds", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
			end
			
			HardFinger.BadSpellHitIDDeath(206612, 0, "ALL", P["EncountersOptions"]["Chronomatic Anomaly"]["announceBurstOfTimeDeaths"], P["EncountersOptions"]["Chronomatic Anomaly"]["trackBurstOfTimeDeaths"], sourceName, destName, spellID, param13, param14, param18)-- Burst of Time
			HardFinger.BadSpellHitIDDeath(205707, 0, "ALL", P["EncountersOptions"]["Chronomatic Anomaly"]["announceTemporalOrb"], P["EncountersOptions"]["Chronomatic Anomaly"]["trackTemporalOrbDeaths"], sourceName, destName, spellID, param13, param14, param18)-- Temporal Orb
			
			-- Trilliax / ID: 
		elseif HardFinger.ActiveEncounter == 1867 or HardFinger.LastEncounter == 1867 then
			if P["EncountersOptions"]["Trilliax"]["trackWhoPulled"] == true then
				HardFinger.WhoPulled("Trilliax", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
			end
			
			-- Annihilation
			HardFinger.BadSpellHitIDDeath(207631, 0, "ALL", P["EncountersOptions"]["Trilliax"]["announceBeamDeaths"], P["EncountersOptions"]["Trilliax"]["trackBeamDeaths"], message, destName, spellID, param13, param14, param18)
			
			if P["EncountersOptions"]["Trilliax"]["trackSterilize"] == true then
				HardFinger.SterilizeUptime(message, sourceName, destName, spellID)
			end
			if P["EncountersOptions"]["Trilliax"]["trackCakes"] == true then
				HardFinger.BadSpellAuraID(214573, -1, true, false, true, message, spellID, destName, param14)-- Stuffed
			end
			if HardFinger.Hpp("boss1") > 30 then
				if P["EncountersOptions"]["Trilliax"]["trackDamage1stImprint"] == true then
					HardFinger.DamageToTarget("Trilliax's Imprint", "First Add - ", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
				end
			else
				if P["EncountersOptions"]["Trilliax"]["trackDamage2ndImprint"] == true then
					HardFinger.DamageToTarget("Trilliax's Imprint", "Second Add - ", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
				end
			end			
			
			-- Spellblade Aluriel / ID: 
		elseif HardFinger.ActiveEncounter == 1871 then
			if P["EncountersOptions"]["Spellblade Aluriel"]["trackWhoPulled"] == true then
				HardFinger.WhoPulled("Spellblade Aluriel", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
			end
			if P["EncountersOptions"]["Spellblade Aluriel"]["trackIceBlock"] == true then
				HardFinger.BadSpellAuraID(213621, -1, nil, false, true, message, spellID, destName, param14) -- Frozen Debuff
			end
			if P["EncountersOptions"]["Spellblade Aluriel"]["damageDoneAddFire"] == true then
				HardFinger.DamageToTarget("Fiery Enchantment", "!", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
			end
			if P["EncountersOptions"]["Spellblade Aluriel"]["damageDoneAddArcane"] == true then
				HardFinger.DamageToTarget("Icy Enchantment", "!", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
			end
			if P["EncountersOptions"]["Spellblade Aluriel"]["damageDoneAddFrost"] == true then
				HardFinger.DamageToTarget("Arcane Enchantment", "!", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
			end
			
			HardFinger.BadSpellHitIDDeath(212494, 0, "ALL", P["EncountersOptions"]["Spellblade Aluriel"]["announceAnnihilateDeaths"], P["EncountersOptions"]["Spellblade Aluriel"]["trackAnnihilateDeaths"], message, destName, destFlags, spellID, param13, param14, param18)-- Annihilated
			
			if P["EncountersOptions"]["Spellblade Aluriel"]["trackAnnihilateDamage"] == true then
				HardFinger.BadSpellDamageID(212494, 0, "Healer|Damager", false, true, message, destName, destFlags, spellID, param13, param14, param18)-- Annihilated
			end
			if P["EncountersOptions"]["Spellblade Aluriel"]["trackPyroInterrupts"] == true then
				HardFinger.HasInterrupt(213281, message, spellID, sourceName, destName) -- Pyroblast Add Cast
			end
			
			-- Tichondrius / ID: 
		elseif HardFinger.ActiveEncounter == 1862 then
			if P["EncountersOptions"]["Tichondrius"]["trackWhoPulled"] == true then
				HardFinger.WhoPulled("Tichondrius", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
			end
			
			-- Krosus / ID: 
		elseif HardFinger.ActiveEncounter == 1842 then
			if P["EncountersOptions"]["Krosus"]["trackWhoPulled"] == true then
				HardFinger.WhoPulled("Krosus", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
			end
			if P["EncountersOptions"]["Krosus"]["trackBeamTimesHit"] == true then
				HardFinger.BadSpellHitID(205391, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param18)-- Fel Beam
			end
			if P["EncountersOptions"]["Krosus"]["trackBeamDeaths"] == true then
				HardFinger.BadSpellHitIDDeath(205391, 0, "ALL", P["EncountersOptions"]["Krosus"]["trackBeamDeathAnnounce"], true, message, destName, destFlags, spellID, param13, param14, param18)-- Fel Beam
			end
			if P["EncountersOptions"]["Krosus"]["trackBurningPitSoak"] == true then
				HardFinger.BadSpellHitID(206376, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param18)-- Burning Spit
			end
			if P["EncountersOptions"]["Krosus"]["trackDamageToAdds"] == true then
				HardFinger.DamageToTarget("Burning Ember", "!", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
			end
			-- Felburst SPELL_DAMAGE 206352
			
			-- High Botanist Tel'arn / ID: 
		elseif HardFinger.ActiveEncounter == 1886 then
			if P["EncountersOptions"]["High Botanist Tel'arn"]["trackWhoPulled"] == true then
				HardFinger.WhoPulled("High Botanist Tel'arn", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
			end
			HardFinger.BadSpellDamageID(218155, 0, "ALL", false, true, message, destName, spellID, param13, param14, param18)-- Solar Collapse
			HardFinger.BadSpellDamageID(218470, 0, "ALL", false, true, message, destName, spellID, param13, param14, param18)-- Controlled Chaos
			HardFinger.BadSpellDamageID(218466, 0, "ALL", false, true, message, destName, spellID, param13, param14, param18)-- Controlled Chaos
			HardFinger.BadSpellDamageID(219235, 0, "ALL", false, true, message, destName, spellID, param13, param14, param18)-- Spores
			
			-- Star Augur Etraeus / ID: 
		elseif HardFinger.ActiveEncounter == 1863 then
			if P["EncountersOptions"]["Star Augur Etraeus"]["trackWhoPulled"] == true then
				HardFinger.WhoPulled("Star Augur Etraeus", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
			end
			
			HardFinger.BadSpellAuraID(207720, -1, true, false, true, message, spellID, destName, param14)-- Witness the Void p3  
			
			-- Grand Magistrix Elisandre / ID: 
		elseif HardFinger.ActiveEncounter == 1872 then
			if P["EncountersOptions"]["Elisandre"]["trackWhoPulled"] == true then
				HardFinger.WhoPulled("Elisandre", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
			end
			if P["EncountersOptions"]["Elisandre"]["trackArcaneRingDamage"] == true then
				HardFinger.BadSpellDamageID(208659, 0, "ALL", false, true, message, destName, spellID, param13, param14, param18)
			end
			HardFinger.BadSpellHitIDDeath(208659, 0, "ALL", P["EncountersOptions"]["Elisandre"]["announceArcaneRingDeaths"], P["EncountersOptions"]["Elisandre"]["trackArcaneRingDeaths"], message, destName, spellID, param13, param14, param18)
			if P["EncountersOptions"]["Elisandre"]["trackAblativeInterrupt"] == true then
				HardFinger.HasInterrupt(209971, message, spellID, sourceName, destName)-- Ablative Pulse 209971
			end			
			-- Gul'dan / ID: 
		elseif HardFinger.ActiveEncounter == 1866 then
			if P["EncountersOptions"]["Gul'dan"]["trackWhoPulled"] == true then
				HardFinger.WhoPulled("Gul'dan", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
			end
			if P["EncountersOptions"]["Gul'dan"]["trackFelEffluxDamage"] == true then
				HardFinger.BadSpellDamageID(206515, 0, "ALL", false, true, message, destName, spellID, param13, param14, param18) -- Fel Efflux
			end
			if P["EncountersOptions"]["Gul'dan"]["trackFelEffluxDeaths"] == true then
				HardFinger.BadSpellHitIDDeath(206515, 0, "ALL", false, true, message, destName, spellID, param13, param14, param18) -- Fel Efflux
			end
			if P["EncountersOptions"]["Gul'dan"]["trackHandOfGuldanDeaths"] == true then
				HardFinger.BadSpellHitIDDeath(212262, 0, "ALL", false, true, message, destName, spellID, param13, param14, param18) -- Hand of Guldan
			end
			HardFinger.BadSpellHitID(227550, 0, "Healer|Damager", P["EncountersOptions"]["Gul'dan"]["announceFelScythe"], P["EncountersOptions"]["Gul'dan"]["trackFelScythe"], message, destName, destFlags, spellID, param13, param14, param18)-- Fel Scythe
			HardFinger.BadSpellHitIDDeath(227550, 3000000, "ALL", true, true, message, destName, destFlags, spellID, param13, param14, param18)-- Fel Scythe
			if P["EncountersOptions"]["Gul'dan"]["trackEmpoweredBondsSoak"] == true then
				HardFinger.BadSpellHitID(206370, 0, "Healer|Damager", false, true, message, destName, destFlags, spellID, param13, param14, param18)-- Empowered Bonds soak
			end
			if P["EncountersOptions"]["Gul'dan"]["trackFelObeliskHit"] == true then
				HardFinger.BadSpellHitID(229948, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param18)-- Fel Obelisk
			end
			if P["EncountersOptions"]["Gul'dan"]["trackDamageToEmpoweredEye"] == true then
				HardFinger.DamageToTarget("Empowered Eye of Gul'dan", "!", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
			end			

		end    

	elseif event == "PLAYER_ENTERING_WORLD" then
		HardFinger.DPrint("Entering World...")
		playerName = UnitName("player")
		HardFinger.CombatStatus = UnitAffectingCombat("player")
		HardFinger.WorldInfo()

	elseif event == "PLAYER_LOGOUT" or event == "PLAYER_LOGIN" then
		HardFinger.SaveVariables()

	elseif event == "ADDON_LOADED" then
		HardFinger.ForceLoad()	
	
	elseif event == "PLAYER_REGEN_ENABLED" then
		HardFinger.CombatStatus = false
		HardFinger.CombatToggle("OUT_OF_COMBAT")

	elseif event == "PLAYER_REGEN_DISABLED" then
		HardFinger.CombatStatus = true
		HardFinger.CombatToggle("IN_COMBAT")

	elseif event == "ENCOUNTER_START" then
		param1, param2 = ...
	    HardFinger.ActiveEncounter = param1
		HardFinger.EncounterTimer = GetTime()
		HardFinger.EncounterName = param2
		HardFinger.timestampDisplayTableTimed = nil
		HardFinger.DPrint(event, encounterID, ...)
		HardFinger.DisplayFailTable("HRESET")
		HardFinger.WorldInfo()

	elseif event == "ENCOUNTER_END" then
	    HardFinger.LastEncounter = HardFinger.ActiveEncounter
		HardFinger.ActiveEncounter = 0
		HardFinger.EncounterName = 0
		HardFinger.DPrint(event, encounterID, ...)
		HardFinger.DPrint("Requesting to display FailTable")
		HardFinger.DisplayFailTable("RESET")
		HardFinger.CombatStatus = false

	elseif event == "CHAT_MSG_ADDON" then
		prefix, message = ...
		if prefix == "D4" and string.find(message, "PT") then
			local index = string.find(message, "\t", 4) - 1
			local timer = string.sub(message, 4, index)
			HardFinger.DPrint(timer+GetTime())
			HardFinger.DBMPullTimer = timer+GetTime()
		end
	end
end

frame:SetScript("OnEvent", OnEvent)
SLASH_HARDFINGER1 = "/hf"
function SlashCmdList.HARDFINGER(msg, editbox)
	msg = string.lower(msg)
	if msg == "" then
		HardFinger.DPrint("\t Type...")
		HardFinger.DPrint("\t|cffff0000/hf stop |r - To pause the addon")
		HardFinger.DPrint("\t|cff00ff00/hf resume |r - To resume the addon")

		local loaded, reason = true
		if (loaded) == true then
			if (HardFingerOptions:IsShown()) then
				HardFingerOptions:Hide();
			else
				HardFinger.UpdateGUIVariables();
				HardFingerOptions:Show();
			end
		else
			AzMsg("Could not open HardFinger Options: |1"..tostring(reason).."|r. Please make sure the addon is enabled from the character selection screen.");
		end
	end
	if msg == "reset" then
		HardFinger.ResetProfile()
	elseif msg == "wp" or msg == "whopulled" then
		P.WhoPulled = HardFinger.toggletf(P.WhoPulled)
		HardFinger.DPrint("HardFinger - WhoPulled is "..HardFinger.StatusText(WhoPulled))
	elseif msg == "bloodlust" or msg == "lust" or msg == "bl" then
		P.BloodLust = HardFinger.toggletf(P.BloodLust)
		HardFinger.DPrint("HardFinger - BloodLust is "..HardFinger.StatusText(BloodLust))
	elseif msg == "ttd" or msg == "time to die" then
		P.TimeToDie = HardFinger.toggletf(P.TimeToDie)
		HardFinger.DPrint("HardFinger - TimeToDie is "..HardFinger.StatusText(TimeToDie))
	elseif msg == "battle rez" or msg == "battlerez" then
		P.BattleRez = HardFinger.toggletf(P.BattleRez)
		HardFinger.DPrint("HardFinger - BattleRez is "..HardFinger.StatusText(BattleRez))
	elseif msg == "taunt" then
		P.Taunt = HardFinger.toggletf(P.Taunt)
		HardFinger.DPrint("HardFinger - Taunt is "..HardFinger.StatusText(Taunt))
	elseif msg == "stop" then
		HardFinger.DPrint("HardFinger - Stopped.")
		P.Status = false
		frame:UnregisterAllEvents()
	elseif msg == "fl" then
		HardFinger.ForceLoad()
		HardFinger.DPrint("Hardfinger - Load forced")
	elseif msg == "sv" then
		HardFinger.SaveVariables()
		HardFinger.DPrint("Hardfinger - Saving Variables")	
	elseif msg == "dft" then
		HardFinger.DPrint("Hardfinger - Displaying Fail Table")
		HardFinger.DisplayFailTable()
	elseif msg == "wft" then
		HardFinger.DPrint("Hardfinger - Wiping Fail Table")
		wipe(HardFinger.FailTable)
	elseif msg == "d" then
		HardFinger.DPrint("HardFinger - Status Debugg")
		for _, v in pairs(DefaultVariables) do
			dbgt = P[v]
			if dbgt == nil then dbgt = "nil" end
			HardFinger.DPrint(v..": "..tostring(dbgt))
		end
	elseif msg == "resume" then
		HardFinger.DPrint("HardFinger - Resumed.")
		P.Status = true
		for _, v in pairs(EVENTS_TO_REGISTER) do
			frame:RegisterEvent(v)
		end
	end
end

-- Variables Loaded
function hf:VARIABLES_LOADED(event)
	isColorBlind = (GetCVar("colorblindMode") == "1");
	-- Default Fonts
	TT_DefaultConfig.fontFace = GameFontNormal:GetFont();
	TT_DefaultConfig.barFontFace = NumberFontNormal:GetFont();
	-- Init Config
	if (not TipTac_Config) then
		TipTac_Config = {};
	end
	cfg = setmetatable(TipTac_Config,{ __index = TT_DefaultConfig });
	-- Default the bar texture if it no longer exists
	bars[1]:SetStatusBarTexture(cfg.barTexture);
	if (not bars[1]:GetStatusBarTexture()) then
		cfg.barTexture = nil;
	end
	-- Hook Tips & Apply Settings
	self:HookTips();
	self:ApplySettings();
	-- Position
	if (cfg.left and cfg.top) then
		self:ClearAllPoints();
		self:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT",cfg.left,cfg.top);
	else
		self:Show();
		-- Just set left and top here, in case the player just closes the anchor without moving, so it doesn't keep showing up when they log in
		cfg.left, cfg.top = self:GetLeft(), self:GetTop();
	end
	-- Cleanup
	self:UnregisterEvent(event);
	self[event] = nil;
end